package za.co.eskom.nrs.xmlvend.client;

import java.text.*;
import java.util.Date;

/**
 * This class generates sequential Message IDs that satisfy the
 * requirements of the XMLVend specification that every message ID be
 * unique within a one-second window. An XMLVend message ID
 * consists of two parts:
 * 
 * - Date/Time
 * - Unique Number
 * 
 * FIXME: This class currently exposes these two parts as separate services, which
 * requires two service calls from client code. If a significant delay exists between
 * these two calls, inconsistent behaviour may arise - this class should rather produce
 * a single "MessageID" instance, which contains the two values internally. In this 
 * regard, the two methods ahve been deprecated.
 * 
 * FIXME: Modify this generator so that it produces a guaranteed unique "UniqueNumber"
 * for every second, but that this uniquenumber counter is reset if the second is
 * no longer the same (i.e. more than one second later). Currently, the counter
 * is simply incremented indefinitely.
 * 
 * NOTE: This class will not generate a unique id the following
 * all transpires within one second:
 * 
 * - This class is loaded in a Java VM, and an ID is requested of it for the first time.
 * - The Java VM is aborted and restarted, and this class is reloaded.
 * - An Id is requested of the class again.
 * 
 * Because this theoretically happens within the same second, and because in both cases 
 * the ID was the first one requested, both may have the same date and number 000001. 
 * This may be partially remedied by starting the count with some random number, 
 * although this scenario is highly unlikely.
 *
 * @author D. Loubser (Solms TCD)
 * @version 0.2 (Jun 28, 2004 - updated Oct 3, 2004)
 */
public class MessageIDGenerator
{
    /** Produces the date part */
    public String getNextDateTime()
    {
    	date = dfmt.format(new Date());
    	return date;
    }
    
    /** Produces the number part */
    public String getNextUniqueNumber()
    {
        String value="";
    	if (date.equals(dfmt.format(new Date())))
        		value = nfmt.format(++counter);
        else
        {	    
        	value = nfmt.format(1);
    			
    		//This makes sure that the the counter is always reset
    		//so that the counter will start from 1 each time that the date is the same. 
    		counter = 1;
        }
        return value;
    }
    
    /**
     * Produces the unique Id which comprises the 
     * date and number.
     * The number is only incremented if the date is the same.
     * For each new date the number will remain as 000001
     */
    public UniqueId getUniqueId()
    {
    	// This creates the UniqueId object to be returned.
    	UniqueId uniqueId = new UniqueId(getNextDateTime(),getNextUniqueNumber());
    	return uniqueId;
    	
    }
    
    // Global "UniqueNumber" counter
    private static int counter=1;
    
    // Used to keep track of the last generated date
    private static String date="";
    
    // Date Format Generator
    private static SimpleDateFormat dfmt = new SimpleDateFormat("yyyyMMddHHmmss");
    
    // Numeric Format
    private static DecimalFormat nfmt = new DecimalFormat("000000");
}
