package za.co.eskom.nrs.xmlvend.client.setup;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import za.co.solms.framework.config.Configuration;
import nu.xom.Document;
import nu.xom.Nodes;
import nu.xom.XPathContext;

/** Maps configuration information from XML to STCD Configuration (properties) */
public class ConfigurationMapper
{
	/** Applies the given structured XML configuration to the given STCD Config */
	public static void applyConfiguration(Document source, Configuration destination)
	{
		// Cycle through all our mappings, and transfer values from XML -> Properties
		for (Entry<String,String> mapping : xpathToConfigMap.entrySet())
		{
			String xpath = mapping.getKey();
			String configKey = mapping.getValue();
			Nodes n = source.query( xpath, xpContext);
			if (n.size() > 0)
			{
				String val = n.get(0).getValue();
				destination.setProperty( configKey, val);
				logger.debug("Set " + configKey + " = " + val);
			}
			else
			{
				// If not in the XMl, remove it from the properties as well
				destination.remove( configKey );
			}
		}
	}
	
	
	private static XPathContext xpContext = new XPathContext("xvc","urn:xmlvend:client:configuration");
	private static Logger logger = Logger.getLogger( ConfigurationMapper.class );
	
	// Maps XPath to pull values out of the XML, to configuration keys
	private static Map<String,String> xpathToConfigMap = new HashMap<String,String>();
	static
	{
		// Client identification
		xpathToConfigMap.put("/xvc:configuration/xvc:client/@clientID","xmlvend.client.clientID");
		xpathToConfigMap.put("/xvc:configuration/xvc:client/@clientIDType","xmlvend.client.clientIDType");
		xpathToConfigMap.put("/xvc:configuration/xvc:client/@terminalID","xmlvend.client.terminalID");
		xpathToConfigMap.put("/xvc:configuration/xvc:client/@terminalIDType","xmlvend.client.terminalIDType");
		// Comms
		xpathToConfigMap.put("/xvc:configuration/xvc:comms/@compressRequests","xmlvend.transport.compression.request");
		xpathToConfigMap.put("/xvc:configuration/xvc:comms/@supportCompressedResponses","xmlvend.transport.compression.response");
		xpathToConfigMap.put("/xvc:configuration/xvc:comms/xvc:ultimateReceiver","xmlvend.transport.service.url");
		// Security (SSL)
		xpathToConfigMap.put("/xvc:configuration/xvc:comms/xvc:security/xvc:keyStoreFile","xmlvend.transport.security.keyStoreFile");
		xpathToConfigMap.put("/xvc:configuration/xvc:comms/xvc:security/xvc:keyStorePassword","xmlvend.transport.security.keyStorePassword");
		// Advice requests
		xpathToConfigMap.put("/xvc:configuration/xvc:advice/xvc:confirm/@auto","xmlvend.advice.autoConfirm");
		xpathToConfigMap.put("/xvc:configuration/xvc:advice/xvc:reversal/@auto","xmlvend.advice.autoReversal");
		// Debugging and Development
		xpathToConfigMap.put("/xvc:configuration/xvc:debug/xvc:messageLoopback/@loopback","xmlvend.debug.messageLoopback.loopback");
	}
}