package za.co.eskom.nrs.xmlvend.client.setup;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.log4j.Logger;

import za.co.solms.framework.config.Configurable;
import za.co.solms.framework.config.Configuration;

/** A filter to place all configuration keys in the HTTP request, prepended with
 * 'configuration.' This effectively makes it easily available to all other
 * components, such as JSPs, etc, whilst not giving them a handle to change
 * it directly.
 * 
 * @author dawidl
 * @version 0.1 (Dec 21, 2005)
 */
public class ConfigurationProviderFilter implements Filter, Configurable
{

	public void init(FilterConfig cfg) throws ServletException
	{
		// Get a handle to the application-wide configuration
		setConfiguration( (Configuration)cfg.getServletContext().
				getAttribute(ClientSetupListener.CONFIGURATION) );
		logger.debug("Initialised" + getConfiguration() == null ? " (null configuration)" : "");
	}

	
	public void doFilter(ServletRequest request, ServletResponse response,
			FilterChain chain) throws IOException, ServletException
	{
		// Set all properties as request attributes (prepended with "configuration.")
		// Can't believe 'Enumeration' is not yet iterable with new 'for' loop!
		Enumeration pnames = getConfiguration().propertyNames();
		while (pnames.hasMoreElements())
		{
			String pname = (String)pnames.nextElement();
			request.setAttribute( CONFIG_PREFIX + pname, config.getProperty(pname) );
		}
		logger.debug("Copied configuration keys to request");
		
		chain.doFilter(request, response);
	}

	
	public void destroy(){}
	
	
	public Configuration getConfiguration()
	{
		return config;
	}
	
	
	public void setConfiguration(Configuration config)
	{
		logger.debug("Configuration set");
		this.config = config;
	}
	
	
	private Configuration config;
	private static final long serialVersionUID = 4182198773512387256L;
	public static final String CONFIG_PREFIX = "configuration.";
	private static final Logger logger = Logger.getLogger(ConfigurationProviderFilter.class);
}
