package za.co.eskom.nrs.xmlvend.client.setup;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.log4j.Logger;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.XPathContext;
import za.co.solms.framework.config.Configurable;
import za.co.solms.framework.config.Configuration;

/** Servlet to serve (GET) and update (POST) client application configuration.
 * 
 * Invoke with: format=[xml|(nothing)] to either get raw XML configuration
 * (which is to be edited and posted back) or to get a UI to edit it. Throws
 * 'InvalidConfigurationException' if a client posts invalid configuration XML.
 * 
 * @author dawidl
 * @version 0.1
 */
public class ConfigurationServlet extends HttpServlet implements Configurable
{
	/** Request the configuration */
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		// What format does the client want it in?
		if ("xml".equals( request.getParameter("format") ))
		{
			logger.debug("Sending configuration in XML format");
			sendXML(request, response);
		}
		else
		{
			logger.debug("Sending user interface to allow editing of configuration");
			sendWebUI(request, response);
		}
	}
	
	
	private void sendXML(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		request.getRequestDispatcher("/admin/_configuration_xml.jspx").forward(request, response);
	}
	
	private void sendWebUI(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		request.getRequestDispatcher("/admin/_configuration_ui.jspx").forward(request, response);
	}
	
	
	/** Submit new configuration */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException ,IOException 
	{
		logger.debug("Configuration update requested, parsing new configuration");
		// Parse the configuration as a XOM
		Document configDoc = null;
		try
		{
			configDoc = xomBuilder.build( request.getInputStream() );
		}
		catch (Exception e)
		{
			logger.error("Invalid configuration (aborting)", e);
			throw new ServletException( new InvalidConfigurationException(e) );
		}
		
		
		// Apply to our configuration
		logger.debug("Invoking configuration mapper to apply new configuration");
		ConfigurationMapper.applyConfiguration( configDoc, getConfiguration() );
		
		// SAVE
		logger.debug("Saving configuration");
		// Save to file
		config.save();
		
		// Apply system properties
		// FIXME: Move this to generic place where it's invoked at startup as well
		// (not just upon update)
		System.setProperty(KEYSTORE_PROPERTY, config.getProperty("xmlvend.transport.security.keyStoreFile"));
		System.setProperty(KEYSTORE_PASSWD_PROPERTY, config.getProperty("xmlvend.transport.security.keyStorePassword"));
		// Debug
		logger.debug("SSL: " + KEYSTORE_PROPERTY + " = " + System.getProperty(KEYSTORE_PROPERTY));
		logger.debug("SSL: " + KEYSTORE_PASSWD_PROPERTY + " = " + System.getProperty(KEYSTORE_PASSWD_PROPERTY));
		// Back to default page
		request.getRequestDispatcher("/").forward(request, response);
	}
	
	
	@Override
	public void init() throws ServletException
	{
		// Get a handle to the application-wide configuration
		setConfiguration( (Configuration)getServletContext().
				getAttribute(ClientSetupListener.CONFIGURATION) );
	}
	
	public Configuration getConfiguration()
	{
		return config;
	}
	
	public void setConfiguration(Configuration config)
	{
		this.config = config;
	}
	
	
	private Builder xomBuilder = new Builder();
	private Configuration config;
	private static final long serialVersionUID = 4182198773512387265L;
	private static Logger logger = Logger.getLogger(ConfigurationServlet.class);
	public static final String KEYSTORE_PROPERTY = "javax.net.ssl.keyStore";
	public static final String KEYSTORE_PASSWD_PROPERTY = "javax.net.ssl.keyStorePassword";
}
