package za.co.eskom.nrs.xmlvend.client.tests;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.ArrayList;
import java.util.Calendar;
import za.co.eskom.nrs.xmlvend.client.MessageIDGenerator;
import junit.framework.TestCase;

/**
 * A test case for the guaranteed message ID generator *
 */
public class MessageIDGeneratorTest extends TestCase {
	protected void setUp() throws Exception {
		super.setUp();
		msgIdGenerator = new MessageIDGenerator();
	}

	protected void tearDown() throws Exception {
		super.tearDown();
		msgIdGenerator = null;
	}

	/*
	 * Test method for
	 * 'za.co.eskom.nrs.xmlvend.client.MessageIDGenerator.getNextDateTime()'
	 */
	public void testGetNextDateTime() {
		// Test length of date
		assertTrue(msgIdGenerator.getNextDateTime().length() == 14);

		// Test format of date
		assertTrue(isValidFormat(msgIdGenerator.getNextDateTime()));
	}

	/*
	 * Test method for
	 * 'za.co.eskom.nrs.xmlvend.client.MessageIDGenerator.getNextUniqueNumber()'
	 */
	public void testGetNextUniqueNumber() {
		// Test length of the unique id
		assertTrue(msgIdGenerator.getNextUniqueNumber().length() == 6);

		// Test that id's are unique when generated within the same second
	assertTrue(generateInSameSecond());

		// Test that id's are the same when generated in different times
		assertTrue(generateInDifferentTimes());
	}

	/*
	 * Test method for
	 * 'za.co.eskom.nrs.xmlvend.client.MessageIDGenerator.getUniqueId()
	 */

	public void testGetUniqueId() {
		// Test that id is 20 characters long????
		//assertTrue(msgIdGenerator.getUniqueId().toString().length() == 20);

		// Test that id is always unique
		assertTrue(isUniqueList());
	}

	public boolean isUniqueList() {
		int counter = 0;
		List<String> idList = new ArrayList<String>();

		// generate 1000 id's 
		while (counter < 1000) {
			counter++;
			idList.add(msgIdGenerator.getUniqueId().toString());
		}

		//and check if all id's are unique
		return isUnique(idList);
	}

	public boolean generateInSameSecond() {
		List<String> idList = new ArrayList<String>();

		// Assumption made that these calendar instances below are
		// executed within less than a minute from each other
		int thisSecond = Calendar.getInstance().get(Calendar.SECOND);
		
		while (thisSecond == Calendar.getInstance().get(Calendar.SECOND)) {					
			idList.add(msgIdGenerator.getNextUniqueNumber());			
		}
		return isUnique(idList);
	}

	public boolean isUnique(List<String> idList) {
		int counter = 0;

		List<String> idSubList = new ArrayList<String>();

		for (String generatedId : idList) {
			
			counter++;
			// the entire list has been traversed
			if (counter == idList.size() - 1)			
				return true;
			
						
			idSubList = idList.subList(counter, idList.size() - 1);

			// for each generated id within the list
			// compare it to every other id in the list
			for (String nextId : idSubList) {
				if (generatedId.equals(nextId))				
					return false;
			}
		}
		return true;
	}

	public boolean isSame(List<String> idList) {
		// assign to first id in the list
		String tempId = idList.get(0);

		for (String generatedId : idList) {
			// the first id must match all the id's in the list
					
			if (!tempId.equals(generatedId))
				return false;			
		}
		return true;
	}

	public boolean generateInDifferentTimes() {
		List<String> idList = new ArrayList<String>();

		int thisSecond = Calendar.getInstance().get(Calendar.SECOND);
		int counter = 0;

		// get 10 id's each within different second
		while (true) {
			if (thisSecond != Calendar.getInstance().get(Calendar.SECOND)) {
				// only increment when time differs
				counter++;
				idList.add(msgIdGenerator.getNextUniqueNumber());
			}

			if (counter == 10)
				break;
		}
		return isSame(idList);
	}

	private boolean isValidFormat(String nextDateTime) {
		// check the format
		DateFormat validFormat = new SimpleDateFormat("yyyyMMddHHmmss");
		try {
			validFormat.parse(nextDateTime);
		} catch (ParseException invalidDate) {
			return false;
		}
		return true;
	}

	public static void main(String[] args) {
		junit.textui.TestRunner.run(MessageIDGeneratorTest.class);
	}

	private MessageIDGenerator msgIdGenerator;
}
