package za.co.eskom.nrs.xmlvend.client.ui;

import java.io.IOException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.el.ELException;
import javax.servlet.jsp.tagext.SimpleTagSupport;
import org.apache.log4j.Logger;
import za.co.eskom.nrs.xmlvend.client.setup.ClientSetupListener;
import za.co.eskom.nrs.xmlvend.client.wsdl.UseCaseMap;
import za.co.eskom.nrs.xmlvend.client.wsdl.UseCaseMap.UseCase;

/** Simple tag to produce an XHTML unordered list of use-case links */
public class UseCaseMenuTag extends SimpleTagSupport
{
	@Override
	public void doTag() throws JspException, IOException
	{
		UseCaseMap useCases = (UseCaseMap)getJspContext().findAttribute( ClientSetupListener.SERVICE_USE_CASES );
		
		// Get a handle to the HTTP request
		HttpServletRequest request = null;
		PageContext pageContext = null;
		try
		{
			pageContext = (PageContext)getJspContext().getVariableResolver().resolveVariable("pageContext");
			request = (HttpServletRequest)pageContext.getRequest();
		} catch (ELException e)
		{
			throw new JspException(e);
		}
		String cpath = request.getContextPath();
		
		// Cycle over use cases and output content
		JspWriter out = getJspContext().getOut();
		out.write("<ul class='useCaseMenu'>");
		for ( UseCase u : useCases.getUseCases())
		{
			String name = u.getName();
			String doc = u.getDocumentation();
			// Format documentation as valid XML
			if (doc != null)
			{
				// TODO Do this more generically
				doc.replace("'","_");
				doc.replace("\"","_");
				doc.replace("&"," and ");
			}
			
			out.write("<li>");
			
			// If the use case is enabled, write a link to the UI
			if (u.isAvailable())
			{
				out.write("<a href='"+ cpath + "/useCase/" + u.getName() + "?in'" +
						(doc != null ? " title='" + doc + "'" : "") +
						">");
			}
			else
			{
				out.write("<span class='disabled'>");
			}
			
				// Body
				out.write(u.getLabel());
			
			if (u.isAvailable())
			{
				out.write("</a>");
			}
			else
			{
				out.write("</span>");
			}
			out.write("</li>");
		}
		out.write("</ul>");
	}
	
	private static Logger logger = Logger.getLogger(UseCaseMenuTag.class);
}
