package za.co.solms.web.xhtml;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;
import org.apache.log4j.Logger;

/** A filter that assists in supporting user agents that do not inherently support
 * XHTML, but are able to render XHTML regardless (by interpreting it as
 * HTML) - this is typical of the MS Internet Explorer browser.
 * 
 * This involves two aspects:
 * 
 *  - Replacement of 'application/xhtml+xml' mime type with 'text/html'
 *  - Removal of XML declaration (TODO Not yet implemented)
 * 
 * @author dawidl
 * @version 0.1 (Jan 4, 2006)
 *
 */
public class MIMEFilter implements Filter
{

	public void init(FilterConfig cfg) throws ServletException
	{
		this.cfg = cfg;
	}

	public void doFilter(ServletRequest req, ServletResponse res,
			FilterChain chain) throws IOException, ServletException
	{
		HttpServletRequest request = (HttpServletRequest)req;
		HttpServletResponse response = (HttpServletResponse)res;
		if ( !acceptsXHTML(request) )
		{
			// Wrap the response in a wrapper that alters mime type as it's set
			logger.debug("User agent does NOT support XHTML");
			response = new MIMEReplacementWrapper( response );
		}
		else
		{
			logger.debug("User agent supports XHTML");
		}
		
		chain.doFilter( request, response );
	}
	
	public boolean acceptsXHTML( HttpServletRequest request )
	{
		String accept = request.getHeader("Accept");
		if (accept != null)
		{
			// TODO Improve with better reqular expression
			return accept.toLowerCase().contains( XHTML_MIME_TYPE );
		}
		// Otherwise assume it does
		return true;
	}

	public void destroy(){}
	
	private FilterConfig cfg;
	public static final String XHTML_MIME_TYPE = "application/xhtml+xml";
	public static final String HTML_MIME_TYPE = "text/html";
	private static Logger logger = Logger.getLogger(MIMEFilter.class);
	
	
	/** Response wrapper to perform MIME replacement (XHTML -> HTML) */
	private class MIMEReplacementWrapper extends HttpServletResponseWrapper
	{
		/** Wraps the given response */
		public MIMEReplacementWrapper(HttpServletResponse response)
		{
			super(response);
		}
		
		
		@Override
		public void setContentType(String mime)
		{
			logger.debug("MIME type set: " + mime);
			if (mime.startsWith( XHTML_MIME_TYPE ))
			{
				logger.debug("Replacing XHTML MIME type with: " + HTML_MIME_TYPE);
				mime = HTML_MIME_TYPE;
			}
			
			super.setContentType(mime);
		}
	}
}
