<?xml version="1.0" encoding="UTF-8"?>
<!-- Produces an XHTML Fragment (in the form of a <div>) of any piece of XML data. 
     This is a simple, generic scheme, and does not incorporate a well-defined 
     customisation layer to allow the selective customisation based on
     the structure of the data being rendered.

    @author D.Loubser (Solms TCD)
    @version 0.1
-->
<xsl:stylesheet 
    version="1.0"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns="http://www.w3.org/1999/xhtml">
    
    <!-- Properly set output format, including DOCTYPE headers, etc for XHTML Strict -->
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    
    <!-- Base (wrapper) element for generated output -->
    <xsl:template match="/" xml:space="preserve">
        <div class="xmlInstance">
            <xsl:apply-templates/>
        </div>
    </xsl:template>
    
    <!-- Match an all complex elements (contains other elements) -->
    <xsl:template match="//*[count(descendant::*) > 0 ]">
        <xsl:call-template name="outputComplexElement">
            <xsl:with-param name="element" select="."/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Match on simple (atomic) elements -->
    <xsl:template match="//*[count(*) = 0]">
        <xsl:call-template name="outputAtomicElement">
            <xsl:with-param name="element" select="."/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Match on attributes -->
    <xsl:template match="@*">
        <xsl:call-template name="outputAttribute">
            <xsl:with-param name="attr" select="."/>
        </xsl:call-template>
    </xsl:template>
    
    <!-- Match on xsi:type attributes -->
    <xsl:template match="@xsi:type">
        <xsl:call-template name="outputTypeAttribute">
            <xsl:with-param name="attr" select="."/>
        </xsl:call-template>
    </xsl:template>
    
    
    <!-- Write a 'complex' element, containing other elements inside it -->
    <xsl:template name="outputComplexElement">
        <xsl:param name="element"/>
        <div class="composite">
            <h2 class="name"><xsl:value-of select="local-name($element)"/></h2>
            
            <!-- Attributes first -->
            <xsl:apply-templates select="@*"/>
            
            <!-- Then any remaining content -->
            <xsl:apply-templates/>
            
        </div>
    </xsl:template>
    
    <!-- Write a simple (atomic) element containing only text, and possibly attributes -->
    <xsl:template name="outputAtomicElement">
        <xsl:param name="element"/>
        <div class="simple">
            <h3 class="name"><xsl:value-of select="local-name($element)"/></h3>
            
            <!-- If we have a value, write it, otherwise indicate that it's empty -->
            <xsl:choose>
                <xsl:when test="string-length($element) > 0">
                    <xsl:value-of select="$element"/>
                </xsl:when>
                <xsl:otherwise>
                    -
                </xsl:otherwise>
            </xsl:choose>

            <!-- See if there are any attributes -->
            <xsl:apply-templates select="@*"/>
            
        </div>
    </xsl:template>
    
    <!-- Write a standard attribute -->
    <xsl:template name="outputAttribute">
        <xsl:param name="attr"/>
        <div class="attribute">
            <h4 class="name"><xsl:value-of select="local-name($attr)"/></h4>
            <xsl:value-of select="$attr"/>
        </div>
    </xsl:template>
    
    <!-- Write a xsi:type standard attribute -->
    <xsl:template name="outputTypeAttribute">
        <xsl:param name="attr"/>
        <div class="typeAttribute">
            <h4 class="name"><xsl:value-of select="local-name($attr)"/></h4>
            <xsl:value-of select="$attr"/>
        </div>
    </xsl:template>
    
</xsl:stylesheet>
